

package komiwojazerbestfs;

import java.util.PriorityQueue;

/**
 * Klasa implementująca metodę podziału i ograniczeń z przeszukiwaniem typu najpierw najlepszy
 * @author Mateusz Łyczek
 */
public class BeFS_BB {
	/** problem do rozwiązania */
	private Komiwojazer problem;

	public BeFS_BB(Komiwojazer p) {
		problem = p;
	}

	/**
	 * Oblicza najlepsze rozwiązanie.
	 * Implementacja metody podziału i ograniczeń z przeszukiwaniem typu najpierw najlepszy
	 * @return obiekt węzła dla problemu komiwojażera z wartościami odpowiadającymi najlepszemu rozwiązaniu
	 */
	public KomiwojazerNode calculate() {
		/** kolejka priorytetowa do przeglądania najpierw najlepszych węzłów */
		PriorityQueue<KomiwojazerNode> Q = new PriorityQueue<KomiwojazerNode>();
		KomiwojazerNode v = new KomiwojazerNode();  // inicjujemy węzeł v na korzeń drzewa
		v.addMiasto(1);                             // dodajemy pierwsze miasto
		v.setGranica(problem.granica(v));           // obliczamy granicę korzenia

		int visited = 1;  // ile węzłów odwiedziliśmy

		int minlength = Integer.MAX_VALUE;          // najlepsza znaleziona dotąd długość ścieżki
		Integer[] bestitems = new Integer[] {};     // najlepsza znaleziona dotąd ścieżka
		Q.add(v);         // dodajemy korzeń do kolejki

		while(!Q.isEmpty()) {
			v = Q.poll(); // wyciągamy najlepszy węzeł z kolejki

			if(v.getGranica() < minlength) {  // jeśli węzeł nadal jest obiecujący
				// przeglądamy wszystkich synów danego węzła, czyli próbujemy dodać do ścieżki następne miasto
				for(int m = 2; m <= problem.getLiczbaMiast(); m++) {
					if(!v.miastoNaSciezce(m)) {  // jeśli to miasto nie jest już na ścieżce
						KomiwojazerNode u = new KomiwojazerNode();
						u.setPoziom(v.getPoziom() + 1);
						u.setSciezka(v.getSciezka());
						u.addMiasto(m);          // dodajemy miasto do ścieżki
						
						System.out.println(u);

						if(u.getPoziom() == problem.getLiczbaMiast() - 1) {  // jeśli to już liść...
							u.addMiasto(1);   // ... to dodaj miasto początkowe

							int dlugosc = problem.length(u);  // sprawdź, czy długość powstałej ścieżki nie jest lepsza
							if(dlugosc < minlength) {
								minlength = dlugosc;
								bestitems = u.getSciezka();
							}
						}
						else {  // jeśli to nie jest liść...
							u.setGranica(problem.granica(u));  // ... policz granicę dla tego węzła
							if(u.getGranica() < minlength) {   // i jeśli jest obiecujący dodaj go do kolejki
								Q.add(u);
							}
						}

						visited++;
					}
				}
			}
		}	

		System.out.println("Odwiedzono: " + visited + " węzłów");

		// konstruujemy węzeł z optymalnym rozwiązaniem
		KomiwojazerNode res = new KomiwojazerNode();
		res.setSciezka(bestitems);
		res.setGranica(minlength);

		return res;
	}
}
