

package komiwojazerbestfs;

/**
 * Klasa implementująca problem komiwojażera
 * @author Mateusz Łyczek
 */
public class Komiwojazer {

	/** liczba miast do odwiedzenia */
	private int liczbaMiast;
	/**
	 * tablica sąsiedztwa przechowująca odległości między miastami
	 * odleglosci[i][j] = odległość od miasta i do j
	 */
	private int[][] odleglosci;

	public Komiwojazer() {
		// konstruujemy prosty problem
		liczbaMiast = 5;

		odleglosci = new int[][] {
					{  0, 14,  4, 10, 20 },
					{ 14,  0,  7,  8,  7 },
					{  4,  5,  0,  7, 16 },
					{ 11,  7,  9,  0,  2 },
					{ 18,  7, 17,  4,  0 }
		};
	}

	/**
	 * Metoda obliczająca granicę dla danego węzła.
	 * Obliczamy ją biorąc sumę minimalnych długości krwawędzi wychodzących z każdego wierzchołka.
	 * Jeśli dany wierzchołek był już odwiedzany, to nie dodajemy minimum dla niego, tylko długość krawędzi,
	 * którą rzeczywiście wyszliśmy z niego.
	 * @param u wierzchołek dla którego obliczamy granicę
	 * @return granica dla danego wierzchołka
	 */
	public float granica(KomiwojazerNode u) {
		float gran = 0f;

		Integer[] path = u.getSciezka();
		int pathLast = path[path.length-1];  // ostatnie miasto na ścieżce

		gran = length(u);   // inicjujemy wartość granicy długością ścieżki w węźle

		for(int m = 1; m <= liczbaMiast; m++) {
			// obliczamy minimum tylko dla miast które nie są na ścieżce (poza ostatnim)
			if(u.miastoNaSciezce(m) && (m != pathLast)) {
				continue;
			}

			int min = Integer.MAX_VALUE;
			for(int i=0; i < odleglosci[m-1].length; i++) {  // przeglądamy wszystkie miasta
				int odl = odleglosci[m-1][i];

				if(odl != 0) {   // rozpotrujemy tylko niezerowe odległości
					/* rozpatrujemy miasto do minimum jeśli to jest pierwsze miasto i nie szukamy tego minimum
					 * dla ostatniego miasta na ścieżce, albo gdy to nie jest pierwsze miasto i nie ma go na ścieżce */
					if(((i == 0) && (m != pathLast)) || !u.miastoNaSciezce(i+1)) {
						if(odl < min) {
							min = odl;
						}
					}
				}
			}

			gran += min;
		}

		return gran;
	}

	/**
	 * Oblicza długość ścieżki dla danego węzła
	 * @param u węzeł do obliczenia ścieżki
	 * @return długość ścieżki
	 */
	public int length(KomiwojazerNode u) {
		Integer[] tour = u.getSciezka();
		int len = 0;

		int from, to;
		for(int i=0; i < tour.length-1; i++) {
			from = tour[i] -1;
			to = tour[i+1] -1;
			
			len += odleglosci[from][to];
		}

		return len;
	}

	/**
	 * Pobiera liczbę miast dla problemu
	 * @return liczba miast
	 */
	public int getLiczbaMiast() {
		return liczbaMiast;
	}
}
