

package komiwojazerbestfs;

import java.util.ArrayList;
import java.util.Arrays;

/**
 * Klasa reprezentująca węzeł drzewa przestrzeni stanów
 * @author Mateusz Łyczek
 */
public class KomiwojazerNode implements Comparable<KomiwojazerNode> {

	/** poziom węzła w drzewie przestrzeni stanów */
	private int poziom;
	/** granica dla węzła */
	private float granica;
	/** lista miast jakie do tej pory odwiedziliśmy */
	private ArrayList<Integer> sciezka;

	public KomiwojazerNode() {
		poziom = 0;
		granica = 0;
		sciezka = new ArrayList<Integer>();
	}

	public float getGranica() {
		return granica;
	}

	public void setGranica(float granica) {
		this.granica = granica;
	}

	public int getPoziom() {
		return poziom;
	}

	public void setPoziom(int poziom) {
		this.poziom = poziom;
	}

	public void addMiasto(int city) {
		sciezka.add(city);
	}

	public void setSciezka(Integer[] s) {
		sciezka.clear();
		sciezka.addAll(Arrays.asList(s));
	}

	public Integer[] getSciezka() {
		return sciezka.toArray(new Integer[] {});
	}

	/**
	 * Określa, czy podane miasto jest na ścieżce obecnego węzła
	 * @param m miasto do sprawdzenia
	 * @return true - jeśli podane miasto jest na ścieżce obecnego węzła, false jeśli nie
	 */
	public boolean miastoNaSciezce(int m) {
		return sciezka.contains(m);
	}
	
	public int compareTo(KomiwojazerNode o) {
		if(granica < o.getGranica()) {
			return -1;
		}
		else if(granica > o.getGranica()) {
			return 1;
		}
		else {
			return 0;
		}
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();

		sb.append(poziom);
		sb.append(") ");

		for(int i=0; i < sciezka.size(); i++) {
			sb.append(sciezka.get(i));

			if(i != sciezka.size() -1) {
				sb.append(" -> ");
			}
		}

		return sb.toString();
	}
}
