

package plecakbfs;

import java.util.LinkedList;
import java.util.Queue;

/**
 * Klasa implementująca metodę podziału i ograniczeń z przeszukiwaniem wszerz
 * @author Mateusz Łyczek
 */
public class BFS_BB {

	/** Problem plecakowy który rozwiązujemy */
	private Plecak problem;

	public BFS_BB(Plecak p) {
		problem = p;
	}

	/**
	 * Oblicza najlepsze rozwiązanie. Implementacja metody podziału i ograniczeń
	 * @return obiekt węzła dla problemu plecakowego z wartościami odpowiadającymi najlepszemu rozwiązaniu
	 */
	public PlecakNode calculate() {
		Queue<PlecakNode> Q = new LinkedList<PlecakNode>();  // kolejka do przeszukiwania wszerz
		PlecakNode v = new PlecakNode();                     // inicjujemy v korzeniem drzewa

		int visited = 1;     // ile węzłów odwiedziliśmy

		int maxprofit = 0;   // najlepsza wartość plecaka jaką udało się nam zobaczyć
		Integer[] bestitems = new Integer[] {};  // rzeczy w plecaku, które dały taką wartośc
		Q.add(v);            // dodajemy korzeń drzewa do kolejki

		while(!Q.isEmpty()) {
			v = Q.poll();    // pobieramy następny wierzchołek z kolejki

			// przeglądamy lewego potomka (zabieramy przedmiot)
			int u_poziom = v.getPoziom() + 1;
			PlecakNode u = new PlecakNode();
			u.setPoziom(u_poziom);
			u.setWaga(v.getWaga() + problem.getWagaFor(u_poziom));
			u.setProfit(v.getProfit() + problem.getPriceFor(u_poziom));
			u.setItems(v.getItems());
			u.addItem(u_poziom);

			/* jeśli wzięcie danej rzeczy nie przekroczyło maksymalnej wagi plecaka i jego wartość
			 * jest teraz lepsza od najlepszej znalezionej do tej pory, to aktualizujemy
			 * odpowiednie zmienne */
			if((u.getWaga() <= problem.getMaxW()) && (u.getProfit() > maxprofit)) {
				maxprofit = u.getProfit();
				bestitems = u.getItems();
			}

			// jeśli węzeł jest obiecujący (czyli ma granicę lepszą niż maxprofit) to dodajemy go do kolejki
			if(problem.granica(u) > maxprofit) {
				Q.add(u);
			}

			// przeglądamy prawego potomka (nie zabieramy przedmiotu)
			PlecakNode u2 = new PlecakNode();
			u2.setPoziom(u_poziom);
			u2.setWaga(v.getWaga());
			u2.setProfit(v.getProfit());
			u2.setItems(v.getItems());

			// jeśli węzeł jest obiecujący (czyli ma granicę lepszą niż maxprofit) to dodajemy go do kolejki
			if(problem.granica(u2) > maxprofit) {
				Q.add(u2);
			}

			// obejrzeliśmy dwa wierzchołki
			visited += 2;
		}

		// tworzymy węzeł z najlepszym rozwiązaniem
		PlecakNode res = new PlecakNode();
		res.setProfit(maxprofit);
		res.setItems(bestitems);

		System.out.println("Odwiedzono: " + visited + " węzłów");

		return res;
	}
}
