

package plecakbestfs;

import java.util.PriorityQueue;

/**
 * Klasa implementująca metodę podziału i ograniczeń z przeszukiwaniem typu najpierw najlepszy
 * @author Mateusz Łyczek
 */
public class BeFS_BB {
	/** problem plecakowy do rozwiązania */
	private Plecak problem;

	public BeFS_BB(Plecak p) {
		problem = p;
	}

	/**
	 * Oblicza najlepsze rozwiązanie.
	 * Implementacja metody podziału i ograniczeń z przeszukiwaniem typu najpierw najlepszy
	 * @return obiekt węzła dla problemu plecakowego z wartościami odpowiadającymi najlepszemu rozwiązaniu
	 */
	public PlecakNode calculate() {
		/** kolejka priorytetowa do przeglądania najpierw najlepszych węzłów */
		PriorityQueue<PlecakNode> Q = new PriorityQueue<PlecakNode>();
		PlecakNode v = new PlecakNode();        // inicjujemy węzeł v na korzeń drzewa
		v.setGranica(problem.granica(v));       // i obliczamy jego granicę

		int visited = 1;  // ile węzłów odwiedziliśmy

		int maxprofit = 0;                      // maksymalna wartość plecaka jaką do tej pory zobaczyliśmy
		Integer[] bestitems = new Integer[] {}; // rzeczy włożone do plecaka, które dały taką wartość
		Q.add(v);         // dodajemy korzeń do kolejki

		while(!Q.isEmpty()) {
			v = Q.poll(); // pobieramy najlepszy do tej pory węzeł

			if(v.getGranica() > maxprofit) {  // sprawdzamy, czy węzeł nadal jest obiecujący

				// przeglądamy lewego potomka (zabieramy przedmiot)
				int u_poziom = v.getPoziom() + 1;
				PlecakNode u = new PlecakNode();
				u.setPoziom(u_poziom);
				u.setWaga(v.getWaga() + problem.getWagaFor(u_poziom));
				u.setProfit(v.getProfit() + problem.getPriceFor(u_poziom));
				u.setItems(v.getItems());
				u.addItem(u_poziom);

				/* jeśli wzięcie danej rzeczy nie przekroczyło maksymalnej wagi plecaka i jego wartość
				 * jest teraz lepsza od najlepszej znalezionej do tej pory, to aktualizujemy
				 * odpowiednie zmienne */
				if((u.getWaga() <= problem.getMaxW()) && (u.getProfit() > maxprofit)) {
					maxprofit = u.getProfit();
					bestitems = u.getItems();
				}

				u.setGranica(problem.granica(u));   // obliczamy i zapamiętujemy w węźle jego granicę
				if(u.getGranica() > maxprofit) {    // jeśli ten węzeł jest obiecujący to dodajemy go do kolejki
					Q.add(u);
				}

				// przeglądamy prawego potomka (nie zabieramy przedmiotu)
				PlecakNode u2 = new PlecakNode();
				u2.setPoziom(u_poziom);
				u2.setWaga(v.getWaga());
				u2.setProfit(v.getProfit());
				u2.setItems(v.getItems());
				u2.setGranica(problem.granica(u2));

				if(u2.getGranica() > maxprofit) {   // jeśli ten węzeł jest obiecujący to dodajemy go do kolejki
					Q.add(u2);
				}

				visited += 2;
			}
		}

		// tworzymy węzeł z najlepszym rozwiązaniem
		PlecakNode res = new PlecakNode();
		res.setProfit(maxprofit);
		res.setItems(bestitems);

		System.out.println("Odwiedzono: " + visited + " węzłów");

		return res;
	}
}
