

package plecakbestfs;

/**
 * Klasa implementuje problem plecakowy
 * @author Mateusz Łyczek
 */
public class Plecak {

	/** ilość przedmiotów */
	private int n;
	/** wagi poszczególnych przedmiotów */
	private int[] w;
	/** ceny poszczególnych przedmiotów */
	private int[] p;
	/** maksymalna waga plecaka */
	private int maxW;

	public Plecak() {
		// kostruujemy przykładowy problem
		n = 4;
		w = new int[] { 2, 5, 10, 5 };
		p = new int[] { 40, 30, 50, 10 };
		maxW = 16;
	}

	/**
	 * Metoda obliczająca granicę dla danego węzła. Zachłannie dobiera kolejne przedmioty
	 * (wg malejących wartości p_i/w_i) do momentu przekroczenia maksymalnej wagi plecaka
	 * @param node Węzeł dla którego obliczamy granicę
	 * @return granica dla danego węzła
	 */
	public float granica(PlecakNode node) {
		PlecakNode u = node;

		if(u.getWaga() > maxW) {   // jeśli przekroczyliśmy wagę to granica jest równa 0
			return 0;
		}
		else {
			float result = u.getProfit();
			int j = u.getPoziom();
			int totweight = u.getWaga(); // tu zliczamy wagę plecaka po dodaniu nowych przedmiotów

			while(j < n && totweight + w[j] <= maxW) {  // zabieramy możliwie dużo przedmiotów
				totweight += w[j];
				result += p[j];
				j++;
			}

			if(j < n) {
				result += (maxW - totweight) * (p[j]/w[j]);   // zabieramy część przedmiotu, który przekroczył wagę
			}

			return result;
		}
	}

	/**
	 * Pobiera wagę i-tego przedmiotu
	 * @param i nr przedmiotu, którego wagę należy pobrać
	 * @return waga dla i-tego przedmiotu
	 */
	public int getWagaFor(int i) {
		int ind = i-1;       // w algorytmie przedmioty numerujemy od 1, tutaj mamy tablicę od 0
		if(ind < w.length) {
			return w[ind];
		}
		else {
			return -1;
		}
	}

	/**
	 * Pobiera wartość i-tego przedmiotu
	 * @param i nr przedmiotu, którego wartość pobrać
	 * @return wartość i-tego przedmiotu
	 */
	public int getPriceFor(int i) {
		int ind = i-1;       // w algorytmie przedmioty numerujemy od 1, tutaj mamy tablicę od 0
		if(ind < p.length) {
			return p[ind];
		}
		else {
			return -1;
		}
	}

	/**
	 * Pobiera maksymalną wagę plecaka
	 * @return maksymalna waga plecaka
	 */
	public int getMaxW() {
		return maxW;
	}
}
