

package plecakbestfs;

import java.util.ArrayList;
import java.util.Arrays;

/**
 * Węzeł dla problemu plecakowego
 * @author Mateusz Łyczek
 */
public class PlecakNode implements Comparable<PlecakNode> {

	/** poziom węzła w drzewie przestrzeni stanów */
	private int poziom;
	/** wartość rzeczy zabranych do plecaka do tej pory */
	private int profit;
	/** waga rzeczy zabranych do plecaka do tej pory */
	private int waga;
	/** granica dla danego węzła */
	private float granica;
	/** lista rzeczy zabranych do plecaka do tej pory */
	private ArrayList<Integer> items;

	public PlecakNode() {
		poziom = 0;
		profit = 0;
		waga = 0;
		items = new ArrayList<Integer>();
	}

	public int getPoziom() {
		return poziom;
	}

	public void setPoziom(int poziom) {
		this.poziom = poziom;
	}

	public int getProfit() {
		return profit;
	}

	public void setProfit(int profit) {
		this.profit = profit;
	}

	public int getWaga() {
		return waga;
	}

	public void setWaga(int waga) {
		this.waga = waga;
	}

	public float getGranica() {
		return granica;
	}

	public void setGranica(float granica) {
		this.granica = granica;
	}

	public void addItem(int item) {
		items.add(item);
	}

	public void setItems(Integer[] itms) {
		items.clear();
		items.addAll(Arrays.asList(itms));
	}

	public Integer[] getItems() {
		return items.toArray(new Integer[] {});
	}

	@Override
	public String toString() {
		return poziom + ") " + waga + "g / " + profit + " zł";
	}

	/**
	 * Metoda porównująca potrzebna do implementacji interfejsu Comparable,
	 * żeby obiekty tej klasy mogły być umieszczane w kolejce priorytetowej
	 * @param o węzeł do porównania
	 * @return -1, 0 lub 1 odpowiednio jeśli granica obecnego węzła jest większa, równa lub mniejsz od granicy o
	 */
	public int compareTo(PlecakNode o) {
		if(granica < o.getGranica()) {
			return 1;
		}
		else if(granica > o.getGranica()) {
			return -1;
		}
		else {
			return 0;
		}
	}
}
